;#include "ti86asm.inc"
#include "ti86.inc"
;#include "crasm86.inc"

version				equ		1						;version ID
max_entries			equ		120						;to conserve space, entries is limited to this
viewonly			equ		0						;flag b0- makes Display only draw view window
imported			equ		1						;flag b1- set if data is unchanged and imported

dynamic_data		equ		$EAD0
flagbyte			equ		dynamic_data+0			;general flags
cursor				equ		dynamic_data+1			;cursor postition (line number)
selected			equ		dynamic_data+2			;selected entry
wind_upper			equ		dynamic_data+3			;entry number of uppermost entry on screen (for scrolling)
tempbyte			equ		dynamic_data+4			;temporary byte for generic storage
tempname			equ		dynamic_data+5			;var-name format storage area
tempentry			equ		dynamic_data+15			;32 byte temp entry space
list_entries		equ		dynamic_data+47			;total entries in list.  Limited to 120.

list				equ		dynamic_data+48			;contains name/number entries. Limited to $0F00 in size.




.org _asm_exec_ram

	nop
	jp ProgStart
	.dw 0
	.dw titletext

ProgStart:
	call _flushallmenus
	call _runindicoff
	ld hl, datafilename-1
	rst 20h							;move to OP1
	rst 10h							;_findsym
	jr nc, Datafile_present			;if c, it doesn't exist, so create it
	ld hl, datafilename-1
	rst 20h
	ld hl, $0002
	call _createStrng				;create the string variable
	ld a, b
	ex de, hl
	call decodeABS
	add hl, de
	add a, 64
	out (5), a
	inc a
	out (6), a
	inc hl
	inc hl							;point to version ID
	ld (hl), $01
	inc hl							;point to number of entries
	ld (hl), $00
	ld a, $0D
	out (5), a						;put call table back
Datafile_present:

top:
	xor a
	ld (flagbyte), a				;zero out the flags
	ld (wind_upper), a				;set upper window entry to first entry
	ld (selected), a				;reset selection counter
	ld (cursor), a					;reset cursor position
	call _clrLCD
	call openList					;get data list from datafile
top_bypass:
	call Display					;display the screen
	jp keyloop						;mainscreen keyloop
	
;---------------------------------------------------------------------


openList:							;get data list from datafile
	ld hl, datafilename-1
	rst 20h
	rst 10h
	ld a, b
	ex de, hl						;ld ahl, bde
	call decodeABS					;decode absolute address
	add hl, de						;de = $4000, add offset into first memory area
	add a, 64						;set bit 6 for RAM page
	out (5), a						;load first area
	inc a
	out (6), a						;load next page, for continuity
	inc hl
	inc hl							;skip length bytes
	inc hl							;skip version byte
	ld a, (hl)						;get entry counter
	ld (list_entries), a			;store it
	cp 0
	jr z, oL_noentries				;don't copy if no data is present
	ld b, a
	ld c, a
	ld a, max_entries
	sub b							;a=max-entries
	jr z, oL_continue				;120 entries is acceptable
	jp p, oL_continue				;less than 120 entries is acceptable. Negative, and its >120, which isn't acceptable.
	ld c, max_entries						;set to max
oL_continue:
	ld a, c							;get entries
	inc hl							;point to start of data
	ld c, 0
	ld b, 5
oL_shiftloop:						;multiply entry counter by 32
	and a							;to reset the carry flag
	rla								;shift a register left
	rl c							;shift c register left with carry from rla
	djnz oL_shiftloop				;do it five times, so I get a*32
	ld b, c							;high byte
	ld c, a							;low byte. BC is now (ENTRIES*32)
	ld de, list						;destination
	ldir							;copy list data out of string variable into LIST
oL_noentries:
	ld a, $0D
	out (5), a						;put the ROM call table back
	ret



Display:							;displays the mainscreen
	call _clrLCD
disp_beginViewW:					;beginning of view window draw code
	ld hl, $FC70
	ld b, 16
disp_topline:						;make a line below title
	ld (hl), $FF
	inc hl
	djnz disp_topline
	ld hl, $FC80
	ld de, 15
	ld b, 49
disp_sidelines:						;draw vertical lines on either side of screen
	ld (hl), %10000000				;leftmost pixel is on
	add hl, de
	ld (hl), %00000001				;rightmost pixel is on
	inc hl							;next row
	djnz disp_sidelines				;do it for [b] rows
	ld b, 16
disp_bottomline:					;draw a line above bottom menu area
	ld (hl), $FF
	inc hl
	djnz disp_bottomline
	ld hl, flagbyte
	bit viewonly, (hl)				;if set, draw view window only
	ret nz							;draw only the view window
disp_top:
	ld hl, $0028
	ld (_pencol), hl
	ld hl, titletext
	call _vputs						;display title
	ld hl, $FC00
	ld b, $70
disp_top_invert:
	ld a, (hl)						;get byte
	cpl								;invert it
	ld (hl), a						;put it back
	inc hl							;next byte
	djnz disp_top_invert			;do that many times
disp_bottom:						;disp the text on the bottom
	ld hl, $3A00
	ld (_pencol), hl
	ld hl, secondtext
	call _vputs
	ld hl, menutext
	call _vputs						;'[2ND] Menu'
	ld hl, $3A60
	ld (_pencol), hl
	ld hl, instr_abouttext
	call _vputs						;'[F1] Help'
disp_cursor:						;display the cursor arrow in correct location
	ld a, (cursor)					;get cursor position
	rla								;*2
	ld b, a							;save
	rla								;*4
	add a, b						;(*2+*4)=*6
	add a, 8						;add to topmost cursor row coord to get coord to disp at
	ld (_penrow), a
	ld a, 2
	ld (_pencol), a
	ld a, 5							;char code for the cursor arrow
	call _vputmap					;display the cursor
disp_entries:						;display the data
	ld a, (wind_upper)
	ld c, a
	ld a, (list_entries)			;get number of entries
	cp 0
	ret z							;don't waste time displaying entries if there are none
	sub c							;total entries - entry at top of window = entries below upper window
	ld c, a							;store this number
	sub 8							;eight entries per screen
	jp m, disp_entries_less8		;if negative, less than a full screen's worth are present
	ld c, 8							;if more than a screenful, we'll display 8
disp_entries_less8:
	ld hl, $0806
	ld (_pencol), hl
	ld a, (wind_upper)				;first entry to display on the screen
	call atimes32					;multiply a by 32, outputs hl
	ld de, list
	add hl, de						;point to the entry referenced in wind_upper
	ld b, c							;number of entries to display
disp_entries_mainloop:				;overall loop that displays up to 8 entries
	push hl
	push bc
	call _vputs						;display name
	pop bc							;contains loop counter
	pop hl							;original entry start point
	ld de, 19
	add hl, de						;point to start of number string
	push hl
	ld c, 0
disp_ent_findNumLength:				;finds out how long (chars) the number string is
	ld a, (hl)
	cp 0
	jr z, disp_ent_findNumdone
	inc c							;increment the character counter
	inc hl							;increment the pointer
	jr disp_ent_findNumLength		;reloop
disp_ent_findNumdone:				;c=length of number string
	ld a, 12
	sub c							;a = (max length - true length)
	rla								;a*2
	rla
	ld c, 78						;leftmost possible col coordinate for displaying the numbers
	add a, c						;determine col coord we should use to right-justify this number
	ld (_pencol), a
	pop hl							;retrieve start of number string
	push hl							;save it
	push bc							;save counter
	call _vputs						;display number
	pop bc
	pop hl
	ld de, 13
	add hl, de						;points to start of next entry
	ld a, 6
	ld (_pencol), a					;reset column coord
	ld a, (_penrow)
	add a, 6
	ld (_penrow), a					;set it to the next line
	djnz disp_entries_mainloop		;do this for all the others
	ret								;done with display





keyloop:							;mainscreen keyloop
	call _getcsc					;get a key (_get_key)
	cp 0
	jr z, keyloop					;if not a real keypress
	cp k_second
	jp z, action_menu				;2ND = menu for selection
	cp k_alpha
	jp z, new						;ALPHA = create new entry
	cp k_more
	jp z, options_menu				;MORE = options menu
	cp k_exit
	jp z, _clrLCD					;EXIT = quit
	cp k_up
	jp z, up						;UP = move cursor up
	cp k_down
	jp z, down						;DOWN = move cursor down
	cp k_del
	jp z, delete					;DEL = delete selected entry
	cp k_xvar
	jp z, edit						;X-VAR = edit
	cp k_f2
	jp z, export					;F2 = Export data
	cp k_f3
	jp z, import					;F3 = Import data
	cp k_f1
	jp z, about						;F1 = About/Help
	jr keyloop


up:
	ld a, (wind_upper)
	ld b, a
	ld a, (cursor)
	cp 0							;is it at uppermost position?
	jr z, up_upper
	ld b, a							;for disp_curonly
	dec a
	ld (cursor), a
	call disp_curonly				;don't refresh whole screen, just update cursor
	jp keyloop
up_upper:
	cp b							;is the uppermost entry displayed also the very first entry in the list?
	jp z, keyloop					;if so, can't do it
	dec b							;scroll up
	ld a, b
	ld (wind_upper), a				;set the new upper limit so it will appear to scroll upward
	call Display
	jp keyloop



down:
	ld a, (list_entries)			;total entries
	cp 0							;no entries?
	jp z, keyloop					;then we can't do anything
	ld b, a
	dec b
	ld a, (cursor)					;current cursor location
	cp b							;are we at the last entry?
	jp z, keyloop					;can't do it then
	cp 7							;lowest position?
	jr z, down_lower				;then we need to scroll
	ld b, a							;for disp_curonly. old cursor position
	inc a
	ld (cursor), a					;update cursor position
	call disp_curonly				;display the cursor but don't refresh whole screen
	jp keyloop						;return to user input loop	
down_lower:							;we need to scroll downward
	ld a, (list_entries)
	ld b, a
	ld a, (wind_upper)				;defines the uppermost entry on screen
	add a, 8						;find lowermost entry number on screen
	ld c, a
	ld a, b
	sub c							;entries-bottommost entry
	jp z, keyloop					;if same, can't do it
	jp m, keyloop					;if negative, can't do it
	ld a, (wind_upper)
	inc a
	ld (wind_upper), a				;this will scroll the view downward
	call Display
	jp keyloop


disp_curonly:						;updates cursor but doesn't refresh entire screen. b=old (cursor) value.
	ld a, b							;get old cursor position
	rla								;*2
	ld b, a							;save
	rla								;*4
	add a, b						;(*2+*4)=*6
	add a, 7						;add to topmost cursor row coord to get coord to disp at
	push af
	inc a
	call d_c_erase
	pop af
	dec a
	call d_c_erase
d_c_newcursor:						;now that old one is erased, draw the new one
	ld a, (cursor)					;get cursor position
	rla								;*2
	ld b, a							;save
	rla								;*4
	add a, b						;(*2+*4)=*6
	add a, 8						;add to topmost cursor row coord to get coord to disp at
	ld (_penrow), a
	ld a, 2
	ld (_pencol), a
	ld a, 5							;char code for the cursor arrow
	call _vputmap					;display the cursor
	ret
d_c_erase:
	ld (_penrow), a
	ld a, 2
	ld (_pencol), a
	ld b, 4							;number of times to display to erase old cursor
d_c_eraseloop:
	ld a, ' '						;char code for SPACE char
	push bc							;save counter
	call _vputmap					;display the cursor
	pop bc
	djnz d_c_eraseloop				;do it [b] times
	ret



action_menu:
	call makeWindow					;make window
	ld hl, $1330
	ld (_pencol), hl
	ld hl, action_menutable			;point to menu data
	call makeMenu					;create a menu in the window
	jr nz, action_menu_cancel		;if exit was pressed
	cp 1
	jp z, new
	cp 2
	jp z, edit
	cp 3
	jp z, delete
action_menu_cancel:
	call destroyWindow				;remove the window
	jp keyloop



options_menu:
	call makeWindow
	ld hl, $1330
	ld (_pencol), hl
	ld hl, options_menutable
	call makeMenu
	jr nz, options_menu_cancel
	cp 1
	jp z, export
	cp 2
	jp z, import
options_menu_cancel:
	call destroyWindow
	jp keyloop



select:									;input- [cursor],[wind_upper]. output- hl=entry, a=entry #, tempentry=data
	ld a, (wind_upper)					;uppermost entry on the screen
	ld b, a
	ld a, (cursor)						;cursor location
	add a, b							;add 'em to get selected entry's number
	push af								;keep the entry number
	call atimes32						;hl=a*32
	ld de, list
	add hl, de
	push hl								;keep that address. It points to start of desired entry in the list.
	ld de, tempentry
	ld bc, 32
	ldir								;copy that entry to temporary entry space
	pop hl								;bring back the entry pointer
	pop af								;entry number
	ret									;we're finished



new:									;make new entry
	ld a, (list_entries)				;current number of entries
	cp max_entries						;is it at the max?
	jr nz, new_continue					;if not, then ok
	call makeWindow
	ld hl, $1330
	ld (_pencol), hl
	ld hl, fileistext
	call _vputs							;'File is '
	ld hl, fulltext
	call _vputs							;'Full'
new_full_keyloop:
	call _getcsc
	cp 0
	jr z, new_full_keyloop
	jp top
new_continue:
	ld hl, flagbyte
	push hl
	set viewonly, (hl)					;this makes Display only draw the view window
	call Display
	pop hl
	res viewonly, (hl)					;undo it
	ld hl, $0000
	ld (_pencol), hl
	ld hl, newtext
	call _vputs							;'New '
	ld hl, entrytext
	call _vputs							;'Entry'
	ld hl, $3A00
	ld (_pencol), hl
	ld hl, enteracceptstext				;'[ENTER] Accepts'
	call _vputs
	ld hl, $3A50
	ld (_pencol), hl
	ld hl, exitcancelstext
	call _vputs							;'[EXIT] Cancel'
	ld hl, $0805
	ld (_pencol), hl
	ld hl, nametext
	call _vputs							;'Name'
	ld hl, $0E08
	ld (_pencol), hl
	ld a, 5
	call _vputmap						;display arrow char
	ld hl, tempentry
	push hl
	ld b, 32
new_clrLoop:							;set tempentry to all zeroes
	ld (hl), 0
	inc hl
	djnz new_clrLoop
	pop hl								;tempentry location
	ld b, 18							;max char limit for name
	call textIn							;get name
	jp nz, top							;if exit was pressed
	ld hl, $1A05
	ld (_pencol), hl
	ld hl, numbertext
	call _vputs							;'number'
	ld hl, $2008
	ld (_pencol), hl
	ld a, 5
	call _vputmap						;disp arrow char as prompt
	ld hl, tempentry+19					;skip name and zterm zero to get address of number
	ld b, 12							;max char limit for number
	call textIn
	jp nz, top							;cancel if exit was pressed
new_create:	
	ld a, (list_entries)				;get total number of entries
	call atimes32						;hl = a*32
	ex de, hl
	ld hl, list							;base address of data list
	add hl, de							;base + offset = end of list, or start of new entry
	ex de, hl							;put it in de
	ld hl, tempentry					;source of new data entry
	ld bc, 32							;length of data entry
	ldir								;add new entry to end of list
	ld a, (list_entries)
	inc a
	ld (list_entries), a				;update entry counter
	call saveList						;save it to the datafile
	call Display
	jp keyloop
	


edit:
	ld a, (list_entries)
	cp 0
	jp z, top_bypass					;can't do anything with an empty list
	ld hl, flagbyte
	push hl
	set viewonly, (hl)					;makes Display only draw the view window
	call Display
	pop hl
	res viewonly, (hl)					;reset it so it displays normally from now on
	ld hl, $0000
	ld (_pencol), hl
	ld hl, edittext
	call _vputs							;'Edit '
	ld hl, entrytext
	call _vputs							;'Entry'
	ld hl, $3A50
	ld (_pencol), hl
	ld hl, exitcancelstext
	call _vputs							;'[EXIT] Cancels'

	ld hl, $0805
	ld (_pencol), hl
	call select							;hl=address of entry, a=number of entry, tempentry=data
	push hl								;save entry address
	ld hl, tempentry
	call _vputs							;display name
	ld hl, $084B
	ld (_pencol), hl
	ld hl, tempentry+19
	call _vputs							;display number
edit_window:
	call makeWindow
	ld hl, $1330
	ld (_pencol), hl
	ld hl, edit_menutable
	call makeMenu
	jr nz, edit_cancel					;if canceled with EXIT
	cp 1
	jr z, edit_name
	cp 2
	jr z, edit_number
	cp 3
	jp z, edit_cancel
edit_cancel:
	pop hl
	jp top

edit_name:
	call destroyWindow					;jp'd from window, so remove the window
	ld hl, tempentry
	ld b, 19
edit_name_clrloop:						;clears name portion of tempentry, leaving number intact
	ld (hl), 0
	inc hl
	djnz edit_name_clrloop
	ld hl, $3A00
	ld (_pencol), hl
	ld hl, enteracceptstext
	call _vputs							;'[ENTER] Accepts'
	ld hl, $1405
	ld (_pencol), hl
	ld hl, nametext
	call _vputs							;'Name:'
	ld hl, $1A08
	ld (_pencol), hl
	ld a, 5
	call _vputmap						;disp the arrow
	ld hl, tempentry					;write to here
	ld b, 18							;max number of chars to allow
	call textin							;get text
	jr nz, edit_window					;if canceled with EXIT
	pop de								;retrieve address of entry
	ld hl, tempentry
	ld bc, 32
	ldir								;copy to the list
	call saveList						;save it to the datafile
	jp edit								;reloop

edit_number:
	call destroyWindow
	ld hl, tempentry+19
	ld b, 13
edit_number_clrLoop:					;clears number part of tempentry
	ld (hl), 0
	inc hl
	djnz edit_number_clrLoop
	ld hl, $3A00
	ld (_pencol), hl
	ld hl, enteracceptstext
	call _vputs							;'[ENTER] Accepts'
	ld hl, $1405
	ld (_pencol), hl
	ld hl, numbertext
	call _vputs							;'Name:'
	ld hl, $1A08
	ld (_pencol), hl
	ld a, 5
	call _vputmap						;disp the arrow
	ld hl, tempentry+19					;write to here
	ld b, 12							;max number of chars to allow
	call textin							;get text
	jp nz, edit_window					;if canceled with EXIT
	pop de								;retrieve address of entry
	ld hl, tempentry
	ld bc, 32
	ldir								;copy to the list
	call saveList						;save it to the datafile
	jp edit								;reloop





delete:									;remove an entry
	ld a, (list_entries)
	cp 0
	jp z, keyloop						;can't delete nothing
	cp 1								;only one left?
	jr z, del_bypassCopy2				;this way it won't crash
	call atimes32						;hl= length of the entire list
	ld de, list
	add hl, de							;end of list
	ex de, hl							;put in de to keep it safe
	push de
	call select							;hl=entry address, a=entry number, tempentry=entry data
	pop de
	push hl								;save entry address
	ex de, hl							;put end back in hl, entry address in de
	and a								;reset carry flag
	sbc hl, de							;find the difference
	ld de, 32
	and a								;reset carry flag
	sbc hl, de							;chop off the entry we will delete
	jr z, del_bypassCopy1				;if zero copy length, ldir will screw up
	ld b, h
	ld c, l								;ld bc, hl.  bc=length to copy
	pop hl								;entry address
	push hl
	add hl, de							;hl=entry+32, the source
	pop de								;de=entry, the destination.  We will copy the remainder of the list backwards.
	ldir								;block copy
del_bypassCopy2:	
	ld a, (list_entries)
	dec a
	ld (list_entries), a				;update the entry counter
	call saveList						;save the list to the datafile
del_up:									;version of UP routine just for exiting DELETE routine
	call Display
	ld a, (wind_upper)
	sub 1
	jp m, up							;if in first screen, we can use normal up routine
	ld (wind_upper), a					;scroll up
	call Display
	jp keyloop
del_bypassCopy1:
	pop hl								;it was pushed, now we need to pop it
	jr del_bypassCopy2



export:
	ld hl, flagbyte
	set viewonly, (hl)					;makes Display only draw the view window
	push hl
	call Display
	pop hl
	res viewonly, (hl)					;makes Display work normally from now on
	ld hl, $0000
	ld (_pencol), hl
	ld hl, exporttext
	call _vputs							;'Export Data String'
	ld hl, $3A50
	ld (_pencol), hl
	ld hl, exitcancelstext
	call _vputs							;'[EXIT] Cancels'
	ld hl, $3A00
	ld (_pencol), hl
	ld hl, enteracceptstext
	call _vputs							;'[ENTER] Accepts'
	ld hl, $0805
	ld (_pencol), hl
	ld hl, stringtotext
	call _vputs							;'String to '
	ld hl, saveastext
	call _vputs							;'Save As:'
	ld hl, $0E08
	ld (_pencol), hl
	ld a, 5
	call _vputmap						;display prompt symbol
	ld hl, tempname
	ld b, 10
exp_tempclear:							;fill tempname with zeroes
	ld (hl), 0
	inc hl
	djnz exp_tempclear
	ld hl, tempname+2					;destination
	ld b, 8								;max chars for a var name
	call textin							;get name from user
	jp nz, top							;if canceled by EXIT press
	ld a, 8								;max chars
	sub b								;b is number of chars less than max that were entered
	ld hl, tempname+1					;name length byte
	ld (hl), a							;save the number of chars entered
	dec hl								;point to beginning of tempname
	rst 20h								;move name to OP1
	rst 10h								;_findsym
	jr nc, exp_exists					;if it exists, ask if they want to overwrite
exp_continue:
	ld hl, tempname
	rst 20h								;put name in OP1 again
	ld a, (list_entries)				;get number of entries
	call atimes32						;HL = a*32
	push hl								;save that length, it is the data length
	ld bc, 6
	add hl, bc							;add for entry file ID, counter byte, version ID. 6 bytes extra.
	call _createStrng					;create the datafile of size [HL] and name [OP1]
	ld a, b
	ex de, hl							;ld ahl, bde
	call decodeABS
	add hl, de							;add $4000 to offset it into first swap area
	add a, 64							;set bit 6
	out (5), a
	inc a
	out (6), a
	inc hl
	inc hl								;skip length bytes
	ex de, hl							;point de to the string variable
	ld hl, file_ID						;source for copy
	ld bc, 4							;file_ID is 4 bytes.  It identifies file as a PhoneBook file.
	ldir								;copy file_ID to string
	ex de, hl							;put pointer to variable back into HL
	ld (hl), version					;version ID byte
	inc hl
	ld a, (list_entries)				;number of entries in the list
	ld (hl), a							;save it to the entry counter byte
	cp 0
	jr z, exp_bypassCopy1				;if zero size, don't ldir, it'll crash
	inc hl								;data starts here
	ex de, hl							;point de to datafile variable
	ld hl, list							;point hl to list data
	pop bc								;retrieve length of list data to copy
	ldir								;copy the list back to the datafile
exp_bypassCopy2:	
	ld a, $0D
	out (5), a							;put back the ROM call table
	jp top								;done
exp_bypassCopy1:
	pop hl								;must pop what we push
	jr exp_bypassCopy2					;skip LDIR
exp_exists:
	call makeWindow
	ld hl, $132D
	ld (_pencol), hl
	ld hl, overwritetext
	call _vputs							;'Overwrite?'
	ld hl, $1930
	ld (_pencol), hl
	ld hl, choice_menutable				;yes or no menu
	call makeMenu						;create a menu
	jp nz, export						;if canceled, restart
	cp 1								;yes?
	jr z, exp_overwrite					;then get rid of original var
	cp 2								;no?
	jp z, export
exp_overwrite:
	ld hl, tempname
	rst 20h
	rst 10h
	call _delvar						;delete variable
	jp exp_continue						;continue with export process




import:
	ld hl, flagbyte
	set viewonly, (hl)					;makes Display only draw the view window
	push hl
	call Display
	pop hl
	res viewonly, (hl)					;makes Display work normally from now on
	ld hl, $0000
	ld (_pencol), hl
	ld hl, importtext
	call _vputs							;'Import Data String'
	ld hl, $3A50
	ld (_pencol), hl
	ld hl, exitcancelstext
	call _vputs							;'[EXIT] Cancels'
	ld hl, $3A00
	ld (_pencol), hl
	ld hl, enteracceptstext
	call _vputs							;'[ENTER] Accepts'
	ld hl, $0805
	ld (_pencol), hl
	ld hl, stringtotext
	call _vputs							;'String to '
	ld hl, loadtext
	call _vputs							;'Load:'
	ld hl, $0E08
	ld (_pencol), hl
	ld a, 5
	call _vputmap						;display prompt symbol
	ld hl, tempname
	ld b, 10
imp_tempclr:							;clear tempname
	ld (hl), 0
	inc hl
	djnz imp_tempclr
	ld hl, tempname+2					;name destination
	ld b, 8								;max chars to allow for a varname
	call textin							;get user input
	jp nz, top							;if canceled with EXIT
	ld a, 8								;max chars
	sub b								;b is remaining chars.  a now = chars accepted
	ld hl, tempname+1					;point to name length byte
	ld (hl), a							;save name length
	dec hl								;point to start of name
	rst 20h
	rst 10h								;_findsym
	jp c, imp_n_exists					;if it does not exist
	ld a, b
	ex de, hl							;ld ahl, bde
	call decodeABS						;decode absolute address in ahl
	add hl, de							;reference address to the first swap area
	add a, 64							;set bit 6 for RAM page
	out (5), a
	inc a
	out (6), a
	ld e, (hl)							;first length byte (lsb)
	inc hl
	ld d, (hl)							;second length byte (msb)
	ld bc, 6
	ex de, hl							;put de into hl
	sbc hl, bc							;subtract six to account for file ID, ver.ID, and entry counter. HL=data length
	jp z, imp_nodata					;can't ldir if file contains zero entries
	jp m, imp_nodata					;if negative, no entries
	ex de, hl							;put length back in de, get data location back in hl
	push de								;save data length
	inc hl								;point to the start of the file ID bytes.  should be [$FF,"PBK"]
	ld a, (hl)
	cp $FF								;check first ID byte
	jr nz, imp_invalid					;if it doesn't match
	inc hl
	ld a, (hl)
	cp 'P'								;check second byte
	jr nz, imp_invalid
	inc hl
	ld a, (hl)
	cp 'B'
	jr nz, imp_invalid
	inc hl
	ld a, (hl)
	cp 'K'
	jr nz, imp_invalid
	inc hl								;point to version ID. In this version, we don't even check it.
	inc hl								;point to entry counter
	ld b, (hl)
	ld a, max_entries					;most entries that are allowed in prog
	sub b
	jr z, imp_continue					;if at the max, ok
	jp p, imp_continue					;if below the max, ok. Otherwise, not ok.
	ld a, $0D
	out (5), a							;first, put the call table back
	pop de								;pop what was pushed
	call makeWindow
	ld hl, $1330
	ld (_pencol), hl
	ld hl, fileistext
	call _vputs							;'File is'
	ld hl, $1930
	ld (_pencol), hl
	ld hl, toolargetext
	call _vputs							;'Too Large'
imp_toolarge_keyloop:
	call _getcsc
	cp 0
	jr z, imp_toolarge_keyloop
	jp top
imp_invalid:
	ld a, $0D
	out (5), a							;put back call table
	pop de								;it was pushed
	call makeWindow
	ld hl, $1330
	ld (_pencol), hl
	ld hl, fileistext
	call _vputs							;'File is'
	ld hl, $133E
	ld (_pencol), hl
	ld hl, invalidtext
	call _vputs							;'Invalid'
	jr imp_toolarge_keyloop				;loop until keypress
imp_continue: 
	ld a, b								;get entry counter into a
	cp 0
	jr z, imp_nodata_pop				;if it has no data entries
	ld (list_entries), a				;store the entry counter
	inc hl								;point to data
	ld de, list							;point to destination
	pop bc								;retrieve the data length
	ldir								;copy into LIST
	ld a, $0D
	out (5), a							;put call table back
	call Display						;show the new list
	call saveList						;save the new list
	jp top								;restart
imp_nodata_pop:
	pop de								;we pushed it earlier
imp_nodata:
	ld a, $0D
	out (5), a							;put call table back
	call makeWindow
	ld hl, $1330
	ld (_pencol), hl
	ld hl, nodatatext
	call _vputs							;'No Data'
	ld hl, $1930
	ld (_pencol), hl
	ld hl, infiletext
	call _vputs							;'In File'
imp_nodataloop:
	call _getcsc
	cp 0
	jr z, imp_nodataloop
	jp top
imp_n_exists:							;if it doesn't exist
	call makeWindow
	ld hl, $1330
	ld (_pencol), hl
	ld hl, notfoundtext
	call _vputs
imp_n_existsloop:
	call _getcsc
	cp 0
	jr z, imp_n_existsloop
	jp top




about:									;Help/About display
	ld hl, flagbyte
	set viewonly, (hl)
	push hl
	call Display
	pop hl
	res viewonly, (hl)
	ld hl, $0002
	ld (_pencol), hl
	ld hl, titletext
	call _vputs							;disp title
	ld hl, $004B
	ld (_pencol), hl
	ld hl, credittext
	call _vputs							;'by Cassady Roop'
	ld hl, $3916
	ld (_pencol), hl
	ld hl, credittext2
	call _vputs							;'SImmoralTech Software'
	ld hl, $FF90
	ld b, 16
about_bottomline:						;redraw bottom line, it was messed up by _vputs
	ld (hl), $FF
	inc hl
	djnz about_bottomline
	ld hl, $FFA0						;row 56
	ld b, $60							;6 rows
about_invertb:							;invert the bottom of the screen
	ld a, (hl)
	cpl									;logical NOT operation
	ld (hl), a
	inc hl
	djnz about_invertb
	ld hl, $FC00
	ld b, $70
about_invertt:							;invert top of screen
	ld a, (hl)
	cpl
	ld (hl), a
	inc hl
	djnz about_invertt
	ld hl, $0905
	ld (_pencol), hl
	ld a, 6
	call _vputmap						;disp an up arrow
	ld hl, $0F05
	ld (_pencol), hl
	ld a, 7
	call _vputmap						;disp a down arrow
	ld hl, $0B10
	ld (_pencol), hl
	ld hl, selecttext
	call _vputs							;'Select/scroll'
	ld hl, $1A03
	ld (_pencol), hl
	ld hl, secondtext
	call _vputs							;'[2ND]'
	ld hl, $1A20
	ld (_pencol), hl
	ld hl, operationstext
	call _vputs							;'Operations'
	ld hl, menutext
	call _vputs							;' Menu'
	ld hl, $2003
	ld (_pencol), hl
	ld hl, moretext
	call _vputs							;'[MORE]'
	ld hl, $2020
	ld (_pencol), hl
	ld hl, optionstext
	call _vputs							;'Options'
	ld hl, menutext
	call _vputs							;' Menu'
	ld hl, $2603
	ld (_pencol), hl
	ld hl, alphatext
	call _vputs							;'[ALPHA]'
	ld hl, $2620
	ld (_pencol), hl
	ld hl, newtext
	call _vputs							;'New '
	ld hl, entrytext
	call _vputs							;'Entry'
	ld hl, $2C03
	ld (_pencol), hl
	ld hl, deltext
	call _vputs							;'[DEL]'
	ld hl, $2C20
	ld (_pencol), hl
	ld hl, deletetext
	call _vputs							;'Delete '
	ld hl, entrytext
	call _vputs							;'Entry'
	ld hl, $3203
	ld (_pencol), hl
	ld hl, exitcancelstext
	ld b, 6
about_exitloop:							;print '[EXIT]'
	push bc
	push hl
	ld a, (hl)
	call _vputmap
	pop hl
	pop bc
	inc hl
	djnz about_exitloop
	ld hl, $3220
	ld (_pencol), hl
	ld hl, quittext
	call _vputs							;'Quit'
about_keyloop:
	call _getcsc
	cp 0
	jr z, about_keyloop
	jp top
	









atimes32:								;multiplies a by 32, outputs HL
	ld h, 0
	ld b, 5
at32_shiftloop:							;multiply a by 32 with h= high byte
	and a								;clear carry flag
	rla
	rl h
	djnz at32_shiftloop
	ld l, a								;hl now = a*32
	ret





makeWindow:								;make window
	ld hl, $FC00
	ld de, _plotSscreen
	ld bc, 1024
	ldir
	set graphDraw, (IY+graphFlags)		;so TI-OS cleans it up later
clrWindow:								;clear small window
	ld hl, $FCF5
	push hl
	ld b, 28
	ld de, 16
clrW_loop1:
	push bc								;preserve counter
	push hl								;preserve start of the line
	ld b, 6								;length of line to clear
clrW_loop2:
	ld (hl), 0
	inc hl
	djnz clrW_loop2
	pop hl								;get start of line we just cleared
	add hl, de							;move to next line
	pop bc								;get counter
	djnz clrW_loop1						;do this for each line
	pop hl								;original upperleft corner byte
	add hl, de
	push hl
	ld (hl), %01111111
	inc hl
	ld b, 4
mw_topline1:
	ld (hl), $FF
	inc hl
	djnz mw_topline1
	ld (hl), %11111110
	pop hl
	add hl, de
	push hl
	ld (hl), %01111111
	inc hl
	ld b, 4
mw_topline2:
	ld (hl), $FF
	inc hl
	djnz mw_topline2
	ld (hl), %11111110	
	pop hl
	add hl, de
	ld de, 5
	ld b, 23
mw_sidelines:
	ld (hl), %01000000
	add hl, de							;forward five bytes
	ld (hl), %00000010
	add hl, de							;ten bytes
	add hl, de							;fifteen bytes
	inc hl								;sixteen. Now on next line
	djnz mw_sidelines					;loop [b] times
	ld (hl), %01111111
	inc hl
	ld b, 4
mw_bottomline:
	ld (hl), $FF
	inc hl
	djnz mw_bottomline
	ld (hl), %11111110
	ret	



	
	
destroyWindow:							;removes window
	ld de, $FC00
	ld hl, _plotSscreen					;saved pic of screen is here
	ld bc, 1024
	ldir
	ret




saveList:								;saves list to datafile
	ld hl, datafilename-1
	rst 20h
	rst 10h
	call _delvar						;delete old list file
	ld hl, datafilename-1
	rst 20h								;put name in OP1 again
	ld a, (list_entries)				;get number of entries
	call atimes32						;HL = a*32
	push hl								;save that length, it is the data length
	inc hl
	inc hl								;add two bytes for entry counter and version ID byte
	call _createStrng					;create the datafile of size [HL] and name [OP1]
	ld a, b
	ex de, hl							;ld ahl, bde
	call decodeABS
	add hl, de							;add $4000 to offset it into first swap area
	add a, 64							;set bit 6
	out (5), a
	inc a
	out (6), a
	inc hl
	inc hl								;skip length bytes
	ld (hl), 1							;version ID
	inc hl
	ld a, (list_entries)				;number of entries in the list
	ld (hl), a							;save it
	cp 0
	jr z, sL_bypassCopy1					;if zero size, don't ldir, it'll crash
	inc hl								;data starts here
	ex de, hl							;point de to datafile variable
	ld hl, list							;point hl to list data
	pop bc								;retrieve length of list data to copy
	ldir								;copy the list back to the datafile
sL_bypassCopy2:	
	ld a, $0D
	out (5), a							;put back the ROM call table
	ret
sL_bypassCopy1:
	pop hl								;must pop what we push
	jr sL_bypassCopy2					;skip LDIR





;decodeABS   decodes absolute address in ahl.
;by Cassady Roop
;input - AHL = absolute address, like that outputted by _findsym
;output - A = page
;		 HL = offset into page.  Add $4000 for first swap window; $8000 for second swap window.
decodeABS:
	cp 0										;is a zero?
	ret z										;if so, hl is already a valid pointer
	dec a
	rlca										;*2
	rlca										;*4
	ld de, $4000								;length of one page
dABSloop:
	push hl										;save
	sbc hl, de									;hl=hl-de
	jr c, dABSdone								;if hl was less than $4000
	pop bc										;get rid of the value we saved
	inc a										;increment the page counter
	jr dABSloop									;reloop
dABSdone:
	inc a										;page correction
	pop hl										;the last pushed value is the offset
	ret											;we are done



;------------------------------------------------------------------------------
;TextIn custom-- Text input that handles upper, lowercase, and numeric chars.
;by Cassady Roop
;INPUT: hl- points to string storage area.
;		b-  max number of characters to accept and store, up to 255.
;OUTPUT: string stored to desired location.
;		 string displayed at pen location on the screen.
;		 if canceled with EXIT key, returns with nz. If successful, returns z.
;		 number of chars less than max accepted returned in b.
;------------------------------------------------------------------------------
TextIn:
	push hl
	push bc
	call _getkey			;get keypress. it destroys hl and bc.
	pop bc
	pop hl
	ld c, a
	cp kenter
	jp z, textin_commit		;enter was pressed
	cp kexit				;exit key?
	jr z, textin_cancel		;EXIT	
	ld a, b					;load counter value
	cp 0					;see if char allowance has maxed out
	jr z, TextIn			;too many characters, so bypass remaining input but not the enter and exit keys.
	ld a, c					;restore keycode to a
	cp kSpace				;is it space?
	jr z, textin_space		;it is a space
	cp ksub					;is it a hyphen?
	jr z, textin_hyphen
	cp kdecPnt				;is it a period?
	jr z, textin_period
	push af					;preserve the keycode
	sub $1C					;k0
	jp m, TextIn_unknown	;it's not a known key if negative
	sub $0A					;k9 +1
	jp m, textin_number		;if neg, assumed to be a number key
	sub $02					;kCapA
	jp m, TextIn_unknown	;it's not a known key if negative
	sub $1A					;kCapZ +1
	jp m, textin_capletter	;if negative, then assume capital letter
	sub $1B					;kz +1
	jp m, textin_lowletter	;if neg, assume lower case letter
textin_unknown:				;go here if it is unknown
	pop af					;what gets pushed, must get popped
	jr TextIn				;must be an unknown key at this point
textin_space:
	ld a, Lspace			;char code for space
	ld (hl), a				;put into string storage
	call _vputmap			;display a space
	inc hl					;forward pointer
	dec b					;decrement counter
	jr TextIn				;re-loop
textin_hyphen:
	ld a, '-'				;char code for hyphen
	ld (hl), a
	call _vputmap
	inc hl
	dec b
	jr textin
textin_period:
	ld a, '.'
	ld (hl), a
	call _vputmap
	inc hl
	dec b
	jr textin
textin_number:
	pop af					;get the original keycode
	add a, 20				;keycode-->char code
	ld (hl), a				;store
	call _vputmap			;display the number
	inc hl					;inc storage pointer
	dec b					;dec counter
	jr TextIn
textin_capletter:
	pop af
	add a, 25				;keycode-->char code
	ld (hl), a				;store
	call _vputmap			;display the letter
	inc hl					;increment string pointer
	dec b					;dec counter
	jr TextIn
textin_lowletter:
	pop af
	add a, 31				;keycode-->char code
	ld (hl), a
	call _vputmap
	inc hl
	dec b
	jr TextIn
textin_commit:				;enter key pressed
	xor a					;a=0
	cp 0					;set the z flag
	ret						;done
textin_cancel:				;exit key pressed
	xor a					;a=0
	cp 1					;res the z flag
	ret						;return with error



;Makemenu
;input- (_pencol/_penrow)
;output- nz if exited, z if successful
;        number of choice returned in a
makemenu:
 ld a, (hl)
 ld (tempbyte), a					;save menutable entries
 ld de, (_pencol)					;base coords
 push de
 ld a, (hl)
 inc hl
 ld b,a
menudisplayloop:
 call _vputs
 ld a,(_penRow)
 add a, 6
 ld (_penRow),a
 pop de
 push de
 ld a, e							;base column coord
 ld (_penCol),a
 djnz menudisplayloop
 pop de
 push de
 ld a, e							;get base column
 sub 4
 ld (_pencol),a
 ld a, d
 ld (_penrow), a
 ld a, 5
 call _vputmap
 ld a, (_pencol)
 sub 4
 ld (_pencol), a
 ld c, 1
menuloop:
 push bc
 call _getcsc
 pop bc
 ld b, 3
 cp K_EXIT
 jp z,menuexit
 cp K_UP
 jr z,m_up
 cp K_DOWN
 jr z,m_down
 cp k_second
 jr z, m_enter
 cp K_ENTER
 jr nz,menuloop
m_enter:
 pop de
 ld a, c
 cp a
 ret
m_down:
 ld a,' '
 push bc
 call _vputmap
 pop bc
 djnz m_down
 ld a, (_penCol)
 sub 3
 ld (_pencol), a
 ld a,(tempbyte)
 rla
 ld b, a
 rla
 add a, b
 pop de
 push de
 add a, d								;add base row
 sub 6
 ld b,a
 ld a,(_penRow)
 cp b
 jr z,setmenu1
 inc c									;increment item counter
 add a, 6
 ld (_penRow),a
 jr menudisp
m_up:
 ld a,' '
 push bc
 call _vputmap
 pop bc
 djnz m_up
 ld a, (_pencol)
 sub 3
 ld (_pencol), a
 ld a,(_penRow)
 pop de
 push de
 cp d
 jr z,setmenumax
 dec c									;decrement item counter
 sub 6
 ld (_penRow),a
 jr menudisp
setmenu1:
 ld c, 1
 ld a, d
 ld (_penRow),a
 jr menudisp
setmenumax:
 ld a,(tempbyte)
 ld c, a
 rla
 ld b, a
 rla
 add a, b
 add a, d
 sub 6
 ld (_penRow),a
menudisp:
 ld a,5
 push bc
 call _vputmap
 pop bc
 ld a, (_pencol)
 sub 4
 ld (_pencol), a
 jp menuloop
menuexit:
 pop de
 ld a, 0
 cp 1
 ret





titletext:
	.db "Phonebook 1.0",0
credittext:
	.db "by Cassady Roop",0
credittext2:
	.db "SImmoralTech Software",0
selecttext:
	.db "Select/Scroll",0
operationstext:
	.db "Operations",0
secondtext:
	.db "[2ND]",0
moretext:
	.db "[MORE]",0
optionstext:
	.db "Options",0
alphatext:
	.db "[ALPHA]",0
entrytext:
	.db " Entry",0
deltext:
	.db "[DEL]",0
menutext:
	.db " Menu",0
instr_abouttext:
	.db "[F1] Help",0
exitcancelstext:
	.db "[EXIT] Cancels",0
enteracceptstext:
	.db "[ENTER] Accepts",0
overwritetext:
	.db "Overwrite?",0
importtext:
	.db "Import",0
exporttext:
	.db "Export",0
stringtotext:
	.db "String to ",0
saveastext:
	.db "Save As:",0
loadtext:
	.db "Load:",0
nametext:
	.db "Name:",0
numbertext:
	.db "Number:",0
notfoundtext:
	.db "Not Found",0
nodatatext:
	.db "No Data",0
infiletext:
	.db "In File",0
quittext:
	.db "Quit",0
fileistext:
	.db "File is ",0
invalidtext:
	.db "Invalid",0
fulltext:
	.db "Full",0
toolargetext:
	.db "Too Large",0


action_menutable:
	.db 3
newtext:
	.db "New",0
edittext:
	.db "Edit",0
deletetext:
	.db "Delete",0

edit_menutable:
	.db 3
	.db "Name",0
	.db "Number",0
	.db "Done",0	

options_menutable:
	.db 2
	.db "Export",0
	.db "Import",0

choice_menutable:
	.db 2
	.db "Yes",0
	.db "No",0

file_ID:							;identifies string as a PhoneBk file
	.db $FF,"PBK"

datafilename:						;name of data file
	.db 8," $PBKdat"


;datafile format:
;offset			purpose
;0				version ID (1 byte)
;1				entries (1 byte)
;2				data (n*32 bytes)

;exported string format:
;offset			purpose
;0				file_ID (4 bytes) Identifies file as a PhoneBook file
;4				version_ID (1 byte) Identifies PhoneBook version that created the file
;5				entries (1 byte) number of entries in data section, 0-[max_entries for that version]
;6				data (entries*32 bytes) the actual data entries

.end
